@@ -13,20 +13,22 @@ use File::Copy;
 use File::Path;
 use Data::Dumper;
 
+
 #=======================================================
+my $gnDebugFlag : shared = 2;		# デバッグレベル指定(0 = まったく出力しない, 1-->大きくなるほど詳細な
+									# デバッグ出力が行われる)
 my $stepflg = 1;
 my @gnReqLibs = ("Compress::Zlib",	"Config",		"Data::Dumper",		"Encode",
-                   "Encode::Guess",	"File::Basename",	"File::Copy",		"File::Path",
+                   "Encode::Guess",	"Fcntl", "File::Basename",	"File::Copy",		"File::Path",
                    "Glib",		"Gtk2",			"Gtk2::GladeXML",	"Gtk2::Pango",
                    "HTML::Entities",	"HTML::TokeParser",	"HTTP::Cookies",	"HTTP::Date",
                    "LWP::MediaTypes",	"LWP::UserAgent",	"List::Util",		"POSIX",
                    "Storable",		"Thread::Queue",	"Time::HiRes",		"Time::Local",
-                   "URI::file",		"threads",		"threads::shared",	"utf8");
+                   "URI::file",		"threads",		"threads::shared",	"utf8", "Fcntl");
 my $gnUserLang;
 my $gnPerlpath;
-my $binpath = &gnGetBinPath;
-my $sharepath = &gnGetLibPath;
-my $manpath = $sharepath . "man\/";
+our $gnGnname = "gnview";		# gnviewの名前
+require(".\/${gnGnname}_lowlevel.pl");
 
 for(;;) {
 # Step 1 (日本語が出力可能であることをチェック)=======================
@@ -48,7 +50,6 @@ for(;;) {
    if($stepflg == 2) {
       &gnprint("==================================\ngnviewで使用するperlを指定します\n");
       
-#      my @pl = `which -a perl 2\>/dev/null`;
       my @pl = &gnWhich(); 
       my $b=0;
       foreach my $a (@pl) {
@@ -86,7 +87,6 @@ for(;;) {
       foreach my $line (@gnPerlopts) {
          chomp($line);
          if($line =~ /useithreads=define/) {
-          #print "$line\n";
           $cnt3_1++;
          }
       }
@@ -97,17 +97,11 @@ for(;;) {
       }else{
       
          # ライブラリのチェック
-#      my @gnview_modules = `cat *.pl | grep -E "^use"|sort|uniq`;
          my $str = 'echo "foreach \$a(@INC) {print \"\$a\\n\";}"';
          my @gnINCs = `$str|$gnPerlpath`;
-      #print "incs:\n" . Dumper(@gnINCs);
          my $cnt3_3 = 0;
          foreach my $eachmodule (@gnReqLibs) {
             chomp($eachmodule);
-         #print "modules\t$eachmodule\n";
-#         $eachmodule =~ s/^use\s+//;
-#         $eachmodule =~ /;/; $eachmodule = $`;
-#            print "\tmodules\t$eachmodule";
             my ${cnt3_2} = 0;
             my $modpath;
             foreach my $inc1(@gnINCs) {
@@ -115,16 +109,13 @@ for(;;) {
                $modpath = $eachmodule;
                $modpath =~ s/::/\//;
                $modpath = $inc1 . "\/" . $modpath . ".pm";
-            #print "\t$modpath\n";
                if(-e $modpath) {
                   ${cnt3_2}++;
-               #print "\t\t\t$modpath\n";
                }
             }
             if(${cnt3_2} == 0) {
                print "\tmodules\t$eachmodule\t\tNOT FOUND\n";
             }else{
-               #print "\t\tfound\(${cnt3_2}\)\n";
                $cnt3_3++;
             }
          
@@ -141,25 +132,63 @@ for(;;) {
       }
    }
 # Step 4 (gnviewのインストールパスを指定)=========
+   # インストール先の設定
+   my $binpath		= &gnGetBinPath;
+   my $sharepath	= &gnGetLibPath;
+   my $manpath		= $sharepath . "man\/";
+   
+   # インストール対象ファイルの設定
+   my @instfile_bin		= ("gnview");
+   my @instfile_share	= ("gnview_firstrun.pl",	"gnview_env.pl",	"gnview_lowlevel.pl",	"1pix.png",
+   							"1pix_glay.png",		"gnview.png",		"gnview.glade",			"gikoNavi.ini",
+   							"url.ini",				"sent.ini");
+   my @instfile_desktop	= ("gnview.desktop");
+   my @instfile_icon	= ("gnview.png");
+   my @instfile_doc		= ("AUTHORS", "COPYING", "ChangeLog", "NEWS", "README", "TODO");
+   my @instfile_man;
+   if ($^O eq "solaris") {
+      @instfile_man = ("gnview.1");
+   }else{
+      @instfile_man = ("gnview.1.gz");
+   }
+   
    if($stepflg == 4) {
          for(;;) {
             &gnprint("\n以下のようにgnviewをインストールします\n");
             &gnprint("インストールパスを確認してください\(インストールパスは変更可能です\)\n");
-            &gnprint("実行ファイルパス\(${binpath}\)\:\n");
-            &gnprint("\tgnview \(実行プログラム\)\n");
-            &gnprint("共有ライブラリパス\(${sharepath}gnview\/\)\:\n");
-            &gnprint("\tgnview_firstrun.pl gnview_env.pl \(実行プログラム\)\n");
-            &gnprint("\t1pix.png 1pix_glay.png gnview.png \(実行プログラムで参照する画像ファイル\)\n");
-            &gnprint("\tgnview.glade gikoNavi.ini url.ini sent.ini \(実行プログラムで参照する設定ファイル\)\n");
-            &gnprint("システム共有パス\(${sharepath}applications\/\)\:\n");
-            &gnprint("\tgnview.desktop \(GUIメニュー用設定ファイル\)\n");
-            &gnprint("システム共有パス\(${sharepath}pixmaps\/\)\:\n");
-            &gnprint("\tgnview.png \(GUIメニュー用画像ファイル\)\n");
-            &gnprint("マニュアルパス\(${manpath}man1\/\)\:\n");
-            &gnprint("\tgnview.1.gz \(マニュアルファイル\)\n");
-            &gnprint("ドキュメントパス\(${sharepath}doc/gnview\/\)\:\n");
-            &gnprint("\tAUTHORS  COPYING  ChangeLog  NEWS  README  TODO \(ドキュメントファイル\)\n");
+
+            &gnprint("実行ファイルパス\(${binpath}\)\:\n\t");
+            #&gnprint("\tgnview \(実行プログラム\)\n");
+            foreach my $fn(@instfile_bin) { print "$fn "; }
+            &gnprint("\(実行プログラム\)\n");
+
+            &gnprint("共有ライブラリパス\(${sharepath}gnview\/\)\:\n\t");
+            #&gnprint("\tgnview_firstrun.pl gnview_env.pl gnview_lowlevel.pl\(実行プログラム\)\n");
+            #&gnprint("\t1pix.png 1pix_glay.png gnview.png \(実行プログラムで参照する画像ファイル\)\n");
+            #&gnprint("\tgnview.glade gikoNavi.ini url.ini sent.ini \(実行プログラムで参照する設定ファイル\)\n");
+            foreach my $fn(@instfile_share) { print "$fn "; }
+            &gnprint("\(実行プログラムから参照されるファイル\)\n");
+
+            &gnprint("システム共有パス\(${sharepath}applications\/\)\:\n\t");
+            #&gnprint("\tgnview.desktop \(GUIメニュー用設定ファイル\)\n");
+            foreach my $fn(@instfile_desktop) { print "$fn "; }
+            &gnprint("\(GUIメニュー用設定ファイル\)\n");
+
+            &gnprint("システム共有パス\(${sharepath}pixmaps\/\)\:\n\t");
+            #&gnprint("\tgnview.png \(GUIメニュー用画像ファイル\)\n");
+            foreach my $fn(@instfile_icon) { print "$fn "; }
+            &gnprint("\(アイコンファイル\)\n");
+
+            &gnprint("ドキュメントパス\(${sharepath}doc/gnview\/\)\:\n\t");
+            #&gnprint("\tAUTHORS  COPYING  ChangeLog  NEWS  README  TODO \(ドキュメントファイル\)\n");
+            foreach my $fn(@instfile_doc) { print "$fn "; }
+            &gnprint("\(ドキュメントファイル\)\n");
             
+            &gnprint("マニュアルパス\(${manpath}man1\/\)\:\n\t");
+            #&gnprint("\tgnview.1.gz \(マニュアルファイル\)\n");
+            foreach my $fn(@instfile_man) { print "$fn "; }
+            &gnprint("\(マニュアルファイル\)\n");
+
             &gnprint("\n以上の内容でインストールを行います。よろしいですか？\[yes\/NO\/beforestep\] ");
             my $yesno = <STDIN>; chomp($yesno);
             $yesno = substr($yesno,0,1);
@@ -221,84 +250,99 @@ for(;;) {
 
       my $uninstinfo = $ENV{'HOME'} . "\/.gnview.uninstinfo";
       open(gnUN, ">$uninstinfo");
-#      my $cmd;
       my $dir;
       my $dirret;
-      my @instfile;
-#      $cmd = "install -D -d $binpath"; system("$cmd");
+      #my @instfile;
+
+      # 実行ファイルのインストール
       $dir = $binpath;
-      $dirret = &gnMkdir($dir, "0755");
-      if($dirret>2) {
+      $dirret = &gnMakeDir($dir, "0755");
+      if($dirret == 0) {
          &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
          exit(1);
       }
-#      $cmd = "install -m 0755 gnview ${binpath}gnview"; system("$cmd");
-      @instfile = ("gnview");
-      &gnInstall(@instfile, "0755", $dir);
-      print gnUN "${binpath}gnview\n";
+      &gnInstall(@instfile_bin, "0755", $dir);
+      foreach my $fn(@instfile_bin) {
+         chomp($fn);
+         print gnUN "$dir$fn\n";
+      }
       
-#      $cmd = "install -D -d ${sharepath}applications"; system("$cmd");
-      $dir = "${sharepath}applications";
-      $dirret = &gnMkdir($dir, "0755");
-      if($dirret>2) {
-         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
-         exit(1);
-      }
-#      $cmd = "install -m 0644 gnview.desktop ${sharepath}applications"; system("$cmd");
-      @instfile = ("gnview.desktop");
-      &gnInstall(@instfile, "0644", $dir);
-      print gnUN "${sharepath}applications\/gnview.desktop\n";
-
-#      $cmd = "install -D -d ${sharepath}pixmaps"; system("$cmd");
-      $dir = "${sharepath}pixmaps";
-      $dirret = &gnMkdir($dir, "0755");
-      if($dirret>2) {
-         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
-         exit(1);
-      }
-#      $cmd = "install -m 0644 gnview.png ${sharepath}pixmaps"; system("$cmd");
-      @instfile = ("gnview.png");
-      &gnInstall(@instfile, "0644", $dir);
-      print gnUN "${sharepath}pixmaps\/gnview.png\n";
-
-#      $cmd = "install -D -d ${sharepath}doc\/gnview"; system("$cmd");
-      $dir = "${sharepath}doc\/gnview";
-      $dirret = &gnMkdir($dir, "0755");
-      if($dirret>2) {
-         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
-         exit(1);
-      }
-#      $cmd = "install -m 0644 AUTHORS  COPYING  ChangeLog  NEWS  README  TODO ${sharepath}doc\/gnview"; system("$cmd");
-      @instfile = ("AUTHORS", "COPYING", "ChangeLog", "NEWS", "README", "TODO");
-      &gnInstall(@instfile, "0644", $dir);
-      print gnUN "${sharepath}doc\/gnview\n";
-
-#      $cmd = "install -D -d ${sharepath}gnview"; system("$cmd");
+      # 共有ファイルのインストール
       $dir = "${sharepath}gnview";
-      $dirret = &gnMkdir($dir, "0755");
-      if($dirret>2) {
+      $dirret = &gnMakeDir($dir, "0755");
+      if($dirret == 0) {
          &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
          exit(1);
       }
-#      $cmd = "install -m 0644 gnview_firstrun.pl gnview_env.pl ${sharepath}gnview"; system("$cmd");
-#      $cmd = "install -m 0644 1pix.png 1pix_glay.png gnview.png ${sharepath}gnview"; system("$cmd");
-#      $cmd = "install -m 0644 gnview.glade gikoNavi.ini url.ini sent.ini ${sharepath}gnview"; system("$cmd");
-      @instfile = ("gnview_firstrun.pl", "gnview_env.pl", "1pix.png", "1pix_glay.png", "gnview.png",
-                      "gnview.glade", "gikoNavi.ini", "url.ini", "sent.ini");
-      &gnInstall(@instfile, "0644", $dir);
-      print gnUN "${sharepath}gnview\n";
+      #@instfile = ("gnview_firstrun.pl", "gnview_env.pl", "gnview_lowlevel.pl", "1pix.png",
+      #             "1pix_glay.png",      "gnview.png",    "gnview.glade",       "gikoNavi.ini",
+      #             "url.ini",            "sent.ini");
+      &gnInstall(@instfile_share, "0644", $dir);
+      foreach my $fn(@instfile_share) {
+         chomp($fn);
+         print gnUN "$dir$fn\n";
+      }
 
-#      $cmd = "install -D -d ${manpath}man1"; system("$cmd");
-      $dir = "${manpath}man1";
-      $dirret = &gnMkdir($dir, "0755");
-      if($dirret>2) {
+      # デスクトップファイルのインストール
+      $dir = "${sharepath}applications";
+      $dirret = &gnMakeDir($dir, "0755");
+      if($dirret == 0) {
          &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
          exit(1);
       }
-#      $cmd = "install -m 0644 gnview.1.gz ${manpath}man1"; system("$cmd");
-      @instfile = ("gnview.1.gz");
-      &gnInstall(@instfile, "0644", $dir);
-      print gnUN "${manpath}man1\/gnview.1.gz\n";
+      #@instfile = ("gnview.desktop");
+      &gnInstall(@instfile_desktop, "0644", $dir);
+      print gnUN "${sharepath}applications\/gnview.desktop\n";
+      foreach my $fn(@instfile_desktop) {
+         chomp($fn);
+         print gnUN "$dir$fn\n";
+      }
+
+      # アイコンファイルのインストール
+      $dir = "${sharepath}pixmaps";
+      $dirret = &gnMakeDir($dir, "0755");
+      if($dirret == 0) {
+         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
+         exit(1);
+      }
+      #@instfile = ("gnview.png");
+      &gnInstall(@instfile_icon, "0644", $dir);
+      foreach my $fn(@instfile_icon) {
+         chomp($fn);
+         print gnUN "$dir$fn\n";
+      }
+
+      # ドキュメントファイルのインストール
+      $dir = "${sharepath}doc\/gnview";
+      $dirret = &gnMakeDir($dir, "0755");
+      if($dirret == 0) {
+         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
+         exit(1);
+      }
+      #@instfile = ("AUTHORS", "COPYING", "ChangeLog", "NEWS", "README", "TODO");
+      &gnInstall(@instfile_doc, "0644", $dir);
+      foreach my $fn(@instfile_doc) {
+         chomp($fn);
+         print gnUN "$dir$fn\n";
+      }
+
+      # マニュアルファイルのインストール
+      $dir = "${manpath}man1";
+      $dirret = &gnMakeDir($dir, "0755");
+      if($dirret == 0) {
+         &gnprint("継続不可能なエラーが発生しました。インストールを中断します\n");
+         exit(1);
+      }
+      #if ($^O eq "solaris") {
+      #   @instfile = ("gnview.1");
+      #}else{
+      #   @instfile = ("gnview.1.gz");
+      #}
+      &gnInstall(@instfile_man, "0644", $dir);
+      foreach my $fn(@instfile_man) {
+         chomp($fn);
+         print gnUN "$dir$fn\n";
+      }
       
       close(gnUN);
       
@@ -308,10 +352,10 @@ for(;;) {
    }
 
 
-if($stepflg > 5) {
-   last;
+   if($stepflg > 5) {
+      last;
+   }
 }
-};
 # 内部関数 ================================================
 
 sub gnGetBinPath {
@@ -325,7 +369,7 @@ sub gnGetBinPath {
          foreach my $dvar(@dvars) {
             chomp($dvar);
             $gnBinPath = $dvar;
-            if(-e "$gnBinPath\/perl") {
+            if(&gnCheckPath($gnBinPath)) {
                return($gnBinPath);
             }
          }
@@ -334,53 +378,23 @@ sub gnGetBinPath {
       foreach my $dvar(@dvars) {
          chomp($dvar);
          $gnBinPath = $dvar;
-         if(-e "$gnBinPath\/perl") {
+         if(&gnCheckPath($gnBinPath)) {
             return($gnBinPath);
          }
       }
       return($gnBinPath);
 }
 
-sub gnGetLibPath {
-   # ライブラリパスをチェック
-   #
-   # 引数:      なし
-   # 返り値: 1: ライブラリへのフルパス
-   
-   my $gnLibPath;
-   if ($^O eq "MSWin32") {
-      # "perl -V"の"osname"がMSWin32の場合は
-      # 実行環境がWindowsであると判断
-      $gnLibPath = "$ENV{'APPDATA'}" . "\\gnview\\";
-   }else{
-      if($ENV{'XDG_DATA_DIRS'}) {
-         my @dvars = split(/:/, $ENV{'XDG_DATA_DIRS'});
-         foreach my $dvar(@dvars) {
-            chomp($dvar);
-            $gnLibPath = $dvar;
-            if(-d "$gnLibPath\/applications") {
-               return($gnLibPath);
-            }
-         }
-      }
-      my @dvars = ('/usr/local/share/', '/usr/share/');
-      foreach my $dvar(@dvars) {
-         chomp($dvar);
-         $gnLibPath = $dvar;
-         if(-d "$gnLibPath\/applications") {
-            return($gnLibPath);
-         }
-      }
-   }
-      return($gnLibPath);
-   
-}
-
 sub gnWhich {
    # 環境変数からperlバイナリの場所を特定する関数
    # 返り値=配列(ここの要素は文字列。perlへのフルパス)
    my @path;
    my @retpath;
+   my $myperl = $^X;
+   
+   if(-e $myperl) {
+      push(@retpath, $myperl);
+   }
    @path = split(/:/, $ENV{'PATH'});
    foreach my $pt(@path) {
       if(-e "$pt\/perl") {
@@ -391,29 +405,6 @@ sub gnWhich {
    
 }
 
-sub gnMkdir {
-   # ディレクトリを作成する関数
-   # 引数=文字列(作成するディレクトリ)
-   # 引数2=文字列(パーミッション。半角数字4文字を想定)
-   # 返り値=integer(1=成功、2=失敗:既にある、3=失敗:作成に失敗)
-   my $dir = $_[0];
-   my $perm = $_[1];
-      if(!(-d $dir)) {
-          my $flg;
-          $flg = mkpath($dir, 0, oct($perm));
-          if ($flg == 0) {
-             &gnprint("ディレクトリの作成に失敗しました: $dir\n");
-             return(3);
-            }else{
-               &gnprint("ディレクトリを作成しました: $dir\n");
-               return(1);
-            }
-      }else{
-         &gnprint("ディレクトリは既に存在します: $dir\n");
-         return(2);
-       }
-}
-
 sub gnInstall {
     # 引数1=配列(ここの要素は文字列。コピーするファイル名が入っていると想定)
     # 引数2=文字列(Permission。ファイルをコピーしたのちにこのパーミッションを割り当てる)
@@ -439,6 +430,62 @@ sub gnprint {
     print $gnPrintStr;
 }
 
+# gnview.plとの一時的なダブり関数 ================================================
+
+sub gnGetLibPath {
+   # ライブラリパスをチェック
+   #
+   # 引数:      なし
+   # 返り値: 1: ライブラリへのフルパス
+   
+   my $gnLibPath = "";
+   if ($^O eq "MSWin32") {
+      # "perl -V"の"osname"がMSWin32の場合は
+      # 実行環境がWindowsであると判断
+      $gnLibPath = &gnGetEnvVal('APPDATA');
+      if(-d $gnLibPath) {
+         if($gnDebugFlag) { print "Your gnview library is $gnLibPath\/gnview\/ from APPDATA environment valiables\n";}
+         return($gnLibPath);
+      }
+   }else{
+	  # Windowsでない場合はFreedesktop.org Specification準拠に従って
+	  # ライブラリパスを探索する。
+	  # ヒットした時点でreturn。
+	  if(&gnGetEnvVal('XDG_DATA_DIRS')) {
+         my @dvars = split(/:/, &gnGetEnvVal('XDG_DATA_DIRS'));
+         foreach my $dvar(@dvars) {
+            chomp($dvar);
+            $gnLibPath = $dvar;
+            if(-d $gnLibPath) {
+               if($gnDebugFlag) { print "Your gnview library is $gnLibPath\/gnview\/ from XDG_DATA_DIRS environment valiables\n";}
+               return($gnLibPath);
+            }
+         }
+      }
+	  my @dvars = ('/usr/local/share/', '/usr/share/');
+      foreach my $dvar(@dvars) {
+         chomp($dvar);
+         $gnLibPath = $dvar;
+         if(-d $gnLibPath) {
+            if($gnDebugFlag) { print "Your gnview library is $gnLibPath\/gnview\/ from fixed path\n";}
+            return($gnLibPath);
+         }
+      }
+   }
+   
+   if($gnLibPath eq "") {
+      print STDERR "Fatal. gnview library path not found. check installation.\n";
+   }else{
+      if(-d $gnLibPath) {
+         if($gnDebugFlag) { print "Your gnview library is $gnLibPath\/gnview\/....but where did you point from \?\n";}
+         return($gnLibPath);
+      }else{
+         print STDERR "Fatal. gnview library path you choiced is not found $gnLibPath\/gnview\/.\n";
+      }
+   }
+}
+
+
 sub env_locale_to_encstr {
     # ロケール名からEncodeモジュールが解釈できる
     # 文字エンコード名を返す関数
@@ -466,18 +513,32 @@ sub env_locale_to_encstr {
      }
     
     # あとはパターンマッチ
-    if    ($myLocStr[1] eq "932") {    return('shiftjis');
-    }elsif($myLocStr[1] eq "pck") { return('shiftjis');
-    }elsif($myLocStr[1] eq "sjis") { return('shiftjis');
-    }elsif($myLocStr[1] eq "20932") { return('euc-jp');
-    }elsif($myLocStr[1] eq "eucjp") { return('euc-jp');
-    }elsif($myLocStr[1] eq "euc") { return('euc-jp');
-    }elsif($myLocStr[1] eq "ujis") { return('euc-jp');
-    }elsif($myLocStr[1] eq "iso-2022-jp") { return('7bit-jis');
-    }elsif($myLocStr[1] eq "utf-8") { return('utf8');
-    }else                           { return('ascii'); # エンコードが判別できなかったらシングルバイトエンコード(単なるバイト列)として解釈する
+    if    ($myLocStr[1] eq "932")			{ return('cp932');
+    }elsif($myLocStr[1] eq "pck")			{ return('cp932');
+    }elsif($myLocStr[1] eq "sjis")			{ return('shiftjis');
+    }elsif($myLocStr[1] eq "20932")		{ return('euc-jp');
+    }elsif($myLocStr[1] eq "eucjp")		{ return('euc-jp');
+    }elsif($myLocStr[1] eq "euc-jp")		{ return('euc-jp');
+    }elsif($myLocStr[1] eq "euc")			{ return('euc-jp');
+    }elsif($myLocStr[1] eq "ujis")			{ return('euc-jp');
+    }elsif($myLocStr[1] eq "iso-2022-jp")	{ return('iso-2022-jp');
+    }elsif($myLocStr[1] eq "utf-8")		{ return('utf8');
+    }elsif($myLocStr[1] eq "utf8")			{ return('utf8');
+    }else										{ return('ascii'); # エンコードが判別できなかったらシングルバイトエンコード(単なるバイト列)として解釈する
     }
 
     
 }
 
+sub gnGetEnvVal {
+   # 環境変数を取得して、UTF-8フラグをつけて返す関数
+   #
+   # 引数：文字列(環境変数のキー)
+   # 返り値：文字列(環境変数の値。Encode::decodeされた値。存在しなければundefが返る)
+   my $key = $_[0];
+   
+   my $val = Encode::decode(&env_locale_to_encstr($ENV{'LANG'}), $ENV{$key});
+   return($val);
+
+}
+
